MODULE TriangToPOV EXPORTS Main;

IMPORT Triang, FileWr, ParseParams, OSError, Process, Wr, Thread;
FROM Triang IMPORT Topology, Coords;
FROM Stdio IMPORT stderr;

TYPE
  Options = RECORD
      inFile: TEXT;
      outFile: TEXT;
      smooth: BOOLEAN;
      normalize: BOOLEAN;
    END;

PROCEDURE DoIt() =
  BEGIN
    WITH 
      o = GetOptions(),
      tc = Triang.Read(o.inFile),
      top = tc.top,
      c = tc.c^
    DO
      IF o.normalize THEN
        Triang.NormalizeVertexNorms(top, c)
      END;
      WritePOVFile(o.outFile, top, c, o.smooth)
    END
  END DoIt;

PROCEDURE WritePOVFile(
    name: TEXT; 
    READONLY top: Topology; 
    READONLY c: Coords; 
    smooth: BOOLEAN;
  ) =
  <* FATAL Wr.Failure, Thread.Alerted, OSError.E *>
  BEGIN
    WITH wr = FileWr.Open(name & ".inc") DO
      Triang.WritePOV(wr, top, c, smooth);
      Wr.Close(wr)
    END
  END WritePOVFile;
 
PROCEDURE GetOptions (): Options =
  <* FATAL Thread.Alerted, Wr.Failure *>
  VAR o: Options;
  BEGIN
    WITH pp = NEW(ParseParams.T).init(stderr) DO
      TRY
        pp.getKeyword("-inFile");                               
        o.inFile := pp.getNext();  

        IF pp.keywordPresent("-outFile") THEN
          o.outFile :=  pp.getNext()
        ELSE
          o.outFile :=  o.inFile
        END;

        o.normalize := pp.keywordPresent("-normalize");

        o.smooth := pp.keywordPresent("-smooth");

        pp.finish();                                       
      EXCEPT                                                            
      | ParseParams.Error =>                                              
          Wr.PutText(stderr, "Usage: TriangToPOV \\\n");
          Wr.PutText(stderr, "  -inFile <name> [ -outFile <name> ]\\\n");
          Wr.PutText(stderr, "  [ -normalize ] [ -smooth ]\n");
          Process.Exit (1);
      END
    END;
    RETURN o
  END GetOptions;
  
BEGIN 
  DoIt();
END TriangToPOV.
 
