MODULE HLR2;

(*
  Created 94-05-04 by J. Stolfi.
  Last edited by stolfi *)

IMPORT LR2, LR3, LR3Extras, LR3x3, Math;

PROCEDURE FromCartesian(READONLY c: LR2.T): Point =
  BEGIN
    RETURN Point{c := LR3.T{1.0D0, c[0], c[1]}}
  END FromCartesian;

PROCEDURE ToCartesian(READONLY p: Point): LR2.T =
  BEGIN
    WITH w = p.c[0] DO
      <* ASSERT w # 0.0D0 *>
      RETURN LR2.T{p.c[1]/w, p.c[2]/w}
    END
  END ToCartesian;

PROCEDURE Side(READONLY p: Point; READONLY L: Line): Sign =
  BEGIN
    WITH s = LR3.Dot(p.c, L.f) DO
      IF s > 0.0D0 THEN
        RETURN 1
      ELSIF s < 0.0D0 THEN
        RETURN -1
      ELSE
        RETURN 0
      END
    END
  END Side;

PROCEDURE Orient(READONLY p, q, r: Point): Sign =
  BEGIN
    WITH s = LR3Extras.Det(p.c, q.c, r.c) DO
      IF s > 0.0D0 THEN
        RETURN 1
      ELSIF s < 0.0D0 THEN
        RETURN -1
      ELSE
        RETURN 0
      END
    END
  END Orient;

PROCEDURE Join(READONLY p, q: Point): Line =
  BEGIN
    RETURN Line{f := LR3Extras.Cross(p.c, q.c)}
  END Join;

PROCEDURE Meet(READONLY K, L: Line): Point =
  BEGIN
    RETURN Point{c := LR3Extras.Cross(K.f, L.f)}
  END Meet;

PROCEDURE MapPoint(READONLY p: Point; READONLY m: PMap): Point =
  BEGIN
    RETURN Point{c := LR3x3.MapRow(p.c, m.dir)}
  END MapPoint;

PROCEDURE MapLine(READONLY L: Line; READONLY m: PMap): Line =
  BEGIN
    RETURN Line{f := LR3x3.MapCol(m.inv, L.f)}
  END MapLine;

PROCEDURE CompMap(READONLY m, n: PMap): PMap =
  BEGIN
    RETURN PMap{
      dir := LR3x3.Mul(m.dir, n.dir),
      inv := LR3x3.Mul(n.inv, m.inv)
    }
  END CompMap;

PROCEDURE Normal(READONLY L: Line): LR2.T =
  BEGIN
    WITH
      nx = L.f[1],
      ny = L.f[2],
      length = Math.hypot(nx, ny)
    DO
      RETURN LR2.T{nx/length, ny/length}
    END;
  END Normal;

PROCEDURE Dir(READONLY frm, tto: Point): LR2.T =
  BEGIN
    WITH
      fw = frm.c[0],
      tw = tto.c[0],

      fx = frm.c[1],
      tx = tto.c[1],
      dx = fw * tx - tw * fx,

      fy = frm.c[2],
      ty = tto.c[2],
      dy = fw * ty - tw * fy,

      length = Math.hypot(dx, dy)
    DO
      RETURN LR2.T{dx/length, dy/length}
    END;
  END Dir;

BEGIN
END HLR2.
