MODULE HR2;

(* 
  Created 94-05-04 by J. Stolfi.
  Last edited by stolfi *)

IMPORT R2, R3, R3Extras, R3x3, Math;

PROCEDURE FromCartesian(READONLY c: R2.T): Point =
  BEGIN
    RETURN Point{c := R3.T{1.0, c[0], c[1]}}
  END FromCartesian;
    
PROCEDURE ToCartesian(READONLY p: Point): R2.T =
  BEGIN
    WITH w = p.c[0] DO
      <* ASSERT w # 0.0 *>
      RETURN R2.T{p.c[1]/w, p.c[2]/w/w}
    END
  END ToCartesian;

PROCEDURE Side(READONLY p: Point; READONLY L: Line): Sign =
  BEGIN
    WITH s = R3.Dot(p.c, L.f) DO
      IF s > 0.0D0 THEN 
        RETURN 1
      ELSIF s < 0.0D0 THEN 
        RETURN -1
      ELSE
        RETURN 0
      END
    END
  END Side;       

PROCEDURE Orient(READONLY p, q, r: Point): Sign =
  BEGIN
    WITH s = R3Extras.Det(p.c, q.c, r.c) DO
      IF s > 0.0D0 THEN 
        RETURN 1
      ELSIF s < 0.0D0 THEN 
        RETURN -1
      ELSE
        RETURN 0
      END
    END
  END Orient;

PROCEDURE Join(READONLY p, q: Point): Line =
  BEGIN
    RETURN Line{f := R3Extras.Cross(p.c, q.c)}
  END Join;   

PROCEDURE Meet(READONLY K, L: Line): Point =
  BEGIN
    RETURN Point{c := R3Extras.Cross(K.f, L.f)}
  END Meet;        

PROCEDURE MapPoint(READONLY p: Point; READONLY m: PMap): Point =
  BEGIN
    RETURN Point{c := R3x3.MapRow(p.c, m.dir)}
  END MapPoint;
        
PROCEDURE MapLine(READONLY L: Line; READONLY m: PMap): Line =
  BEGIN
    RETURN Line{f := R3x3.MapCol(m.inv, L.f)}
  END MapLine;
    
PROCEDURE CompMap(READONLY m, n: PMap): PMap =
  BEGIN
    RETURN PMap{
      dir := R3x3.Mul(m.dir, n.dir),
      inv := R3x3.Mul(n.inv, m.inv)
    }
  END CompMap;    

PROCEDURE Normal(READONLY L: Line): R2.T =
  BEGIN
    WITH 
      nx = FLOAT(L.f[1], LONGREAL),
      ny = FLOAT(L.f[2], LONGREAL),
      
      length = Math.hypot(nx, ny)
    DO
      RETURN R2.T{FLOAT(nx/length), FLOAT(ny/length)}
    END;
  END Normal;    

PROCEDURE Dir(READONLY frm, tto: Point): R2.T =
  BEGIN
    WITH 
      fw = FLOAT(frm.c[0], LONGREAL),
      tw = FLOAT(tto.c[0], LONGREAL),
      
      fx = FLOAT(frm.c[1], LONGREAL),
      tx = FLOAT(tto.c[1], LONGREAL),
      dx = fw * tx - tw * fx,
      
      fy = FLOAT(frm.c[2], LONGREAL),
      ty = FLOAT(tto.c[2], LONGREAL),
      dy = fw * ty - tw * fy,
      
      length = Math.hypot(dx, dy)
    DO
      RETURN R2.T{FLOAT(dx/length), FLOAT(dy/length)}
    END;
  END Dir;    

BEGIN 
END HR2.
