MODULE HR3;

(* 
  Created 93-05-18 by Marcos C. Carrard.
  Based on H3.pas by J. Stolfi.
  Last edited by stolfi *)

IMPORT R3, R3Extras, R4, R4Extras, R6, R4x4, Math;

PROCEDURE FromCartesian(READONLY c: R3.T): Point =
  BEGIN
    RETURN Point{c := R4.T{1.0, c[0], c[1], c[2]}}
  END FromCartesian;
    
PROCEDURE ToCartesian(READONLY p: Point): R3.T =
  BEGIN
    WITH w = p.c[0] DO
      <* ASSERT w # 0.0 *>
      RETURN R3.T{p.c[1]/w, p.c[2]/w, p.c[3]/w}
    END
  END ToCartesian;

PROCEDURE Side(READONLY p: Point; READONLY Q: Plane): Sign =
  BEGIN
    WITH s = R4.Dot(p.c, Q.f) DO
      IF s > 0.0D0 THEN 
        RETURN 1
      ELSIF s < 0.0D0 THEN 
        RETURN -1
      ELSE
        RETURN 0
      END
    END
  END Side;       

PROCEDURE Orient(READONLY p, q, r, s: Point): Sign =
  BEGIN
    WITH s = R4Extras.Det(p.c, q.c, r.c, s.c) DO
      IF s > 0.0D0 THEN 
        RETURN 1
      ELSIF s < 0.0D0 THEN 
        RETURN -1
      ELSE
        RETURN 0
      END
    END
  END Orient;
  
PROCEDURE LineFromTwoPoints(READONLY p, q: Point): Line =
  BEGIN
    WITH
      a0 = FLOAT(p.c[0], LONGREAL),
      a1 = FLOAT(p.c[1], LONGREAL),
      a2 = FLOAT(p.c[2], LONGREAL),
      a3 = FLOAT(p.c[3], LONGREAL),
      
      b0 = FLOAT(q.c[0], LONGREAL),
      b1 = FLOAT(q.c[1], LONGREAL),
      b2 = FLOAT(q.c[2], LONGREAL),
      b3 = FLOAT(q.c[3], LONGREAL)
    DO
      RETURN Line{
        k := R6.T{
          FLOAT(a0*b1 - a1*b0),
          FLOAT(a0*b2 - a2*b0),
          FLOAT(a1*b2 - a2*b1),
          FLOAT(a0*b3 - a3*b0),
          FLOAT(a1*b3 - a3*b1),
          FLOAT(a2*b3 - a3*b2)
        }
      }
    END
  END LineFromTwoPoints;
  
PROCEDURE PlaneFromThreePoints(READONLY p, q, r: Point): Plane =
  BEGIN
    RETURN Plane{f := R4Extras.Cross(p.c, q.c, r.c)}
  END PlaneFromThreePoints;   

PROCEDURE PlaneFromLineAndPoint(READONLY n: Line; READONLY r: Point): Plane =
  BEGIN
    WITH
      a01 = FLOAT(n.k[0], LONGREAL),
      a02 = FLOAT(n.k[1], LONGREAL),
      a12 = FLOAT(n.k[2], LONGREAL),
      a03 = FLOAT(n.k[3], LONGREAL),
      a13 = FLOAT(n.k[4], LONGREAL),
      a23 = FLOAT(n.k[5], LONGREAL),
      
      b0 = FLOAT(r.c[0], LONGREAL),
      b1 = FLOAT(r.c[1], LONGREAL),
      b2 = FLOAT(r.c[2], LONGREAL),
      b3 = FLOAT(r.c[3], LONGREAL)
    DO
      RETURN Plane{
        f := R4.T{
          FLOAT(- a12*b3 + a13*b2 - a23*b1),
          FLOAT(+ a02*b3 - a03*b2 + a23*b0),
          FLOAT(- a01*b3 + a03*b1 - a13*b0),
          FLOAT(+ a01*b2 - a02*b1 + a12*b0)
        }
      }
    END
  END PlaneFromLineAndPoint;

PROCEDURE LineFromTwoPlanes(READONLY P, Q: Plane): Line =
  BEGIN
    WITH
      a0 = FLOAT(P.f[0], LONGREAL),
      a1 = FLOAT(P.f[1], LONGREAL),
      a2 = FLOAT(P.f[2], LONGREAL),
      a3 = FLOAT(P.f[3], LONGREAL),
      
      b0 = FLOAT(Q.f[0], LONGREAL),
      b1 = FLOAT(Q.f[1], LONGREAL),
      b2 = FLOAT(Q.f[2], LONGREAL),
      b3 = FLOAT(Q.f[3], LONGREAL)
    DO
      RETURN Line{
        k := R6.T{
          FLOAT(+ a2*b3 - a3*b2),
          FLOAT(- a1*b3 + a3*b1),
          FLOAT(+ a0*b3 - a3*b0),
          FLOAT(+ a1*b2 - a2*b1),
          FLOAT(- a0*b2 + a2*b0),
          FLOAT(+ a0*b1 - a1*b0)
        }
      }
    END
  END LineFromTwoPlanes;
  
PROCEDURE PointFromThreePlanes(READONLY P, Q, R: Plane): Point =
  BEGIN
    RETURN Point{c := R4Extras.Cross(Q.f, P.f, R.f)}
  END PointFromThreePlanes; 
  
PROCEDURE PointFromLineAndPlane(READONLY n: Line; R: Plane): Point =
  BEGIN
    WITH
      a01 = FLOAT(n.k[0], LONGREAL),
      a02 = FLOAT(n.k[1], LONGREAL),
      a12 = FLOAT(n.k[2], LONGREAL),
      a03 = FLOAT(n.k[3], LONGREAL),
      a13 = FLOAT(n.k[4], LONGREAL),
      a23 = FLOAT(n.k[5], LONGREAL),
      
      b123 = FLOAT(R.f[0], LONGREAL),
      b023 = FLOAT(R.f[1], LONGREAL),
      b013 = FLOAT(R.f[2], LONGREAL),
      b012 = FLOAT(R.f[3], LONGREAL)
    DO
      RETURN Point{
        c := R4.T{
          FLOAT(+ a01*b023 + a02*b013 + a03*b012),
          FLOAT(- a01*b123 + a12*b013 + a13*b012),
          FLOAT(- a02*b123 - a12*b023 + a23*b012),
          FLOAT(- a03*b123 - a13*b023 - a23*b013)
        }
      }
    END
  END PointFromLineAndPlane;

PROCEDURE MapPoint(READONLY p: Point; READONLY m: PMap): Point =
  BEGIN
    RETURN Point{c := R4x4.MapRow(p.c, m.dir)}
  END MapPoint;
        
PROCEDURE InvMapPoint(READONLY p: Point; READONLY m: PMap): Point =
  BEGIN
    RETURN Point{c := R4x4.MapRow(p.c, m.inv)}
  END InvMapPoint;
        
PROCEDURE MapPlane(READONLY P: Plane; READONLY m: PMap): Plane =
  BEGIN
    RETURN Plane{f := R4x4.MapCol(m.inv, P.f)}
  END MapPlane;
    
PROCEDURE InvMapPlane(READONLY P: Plane; READONLY m: PMap): Plane =
  BEGIN
    RETURN Plane{f := R4x4.MapCol(m.dir, P.f)}
  END InvMapPlane;
    
PROCEDURE CompMap(READONLY m, n: PMap): PMap =
  BEGIN
    RETURN PMap{
      dir := R4x4.Mul(m.dir, n.dir),
      inv := R4x4.Mul(n.inv, m.inv)
    }
  END CompMap;    
  
PROCEDURE InvMap(READONLY m: PMap): PMap =
  BEGIN
    RETURN PMap{dir := m.inv, inv := m.dir}
  END InvMap;

PROCEDURE Normal(READONLY P: Plane): R3.T =
  BEGIN
    WITH 
      nx = FLOAT(P.f[1], LONGREAL),
      ny = FLOAT(P.f[2], LONGREAL),
      nz = FLOAT(P.f[3], LONGREAL),
      
      length = Math.hypot(Math.hypot(nx, ny), nz)
    DO
      RETURN R3.T{FLOAT(nx/length), FLOAT(ny/length), FLOAT(nz/length)}
    END;
  END Normal;    

PROCEDURE Dir(READONLY frm, tto: Point): R3.T =
  BEGIN
    WITH 
      fw = FLOAT(frm.c[0], LONGREAL),
      tw = FLOAT(tto.c[0], LONGREAL),
      
      fx = FLOAT(frm.c[1], LONGREAL),
      tx = FLOAT(tto.c[1], LONGREAL),
      dx = fw * tx - tw * fx,
      
      fy = FLOAT(frm.c[2], LONGREAL),
      ty = FLOAT(tto.c[2], LONGREAL),
      dy = fw * ty - tw * fy,
      
      fz = FLOAT(frm.c[3], LONGREAL),
      tz = FLOAT(tto.c[3], LONGREAL),
      dz = fw * tz - tw * fz,
      
      length = Math.hypot(Math.hypot(dx, dy), dz)
    DO
      RETURN R3.T{FLOAT(dx/length), FLOAT(dy/length), FLOAT(dz/length)}
    END;
  END Dir;  
  
PROCEDURE Dist(READONLY a, b: Point): LONGREAL =
  BEGIN
    WITH 
      aw = 1.0d0/FLOAT(a.c[0], LONGREAL),
      bw = 1.0d0/FLOAT(b.c[0], LONGREAL),
      
      ax = FLOAT(a.c[1], LONGREAL),
      bx = FLOAT(b.c[1], LONGREAL),
      dx = ax*aw - bx*bw,
      
      ay = FLOAT(a.c[2], LONGREAL),
      by = FLOAT(b.c[2], LONGREAL),
      dy = ay*aw - by*bw,
      
      az = FLOAT(a.c[3], LONGREAL),
      bz = FLOAT(b.c[3], LONGREAL),
      dz = az*aw - bz*bw
    DO
     RETURN Math.hypot(Math.hypot(dx, dy), dz)
    END;
  END Dist;
    
PROCEDURE DistSqr(READONLY a, b: Point): LONGREAL =
  BEGIN
    WITH 
      aw = 1.0d0/FLOAT(a.c[0], LONGREAL),
      bw = 1.0d0/FLOAT(b.c[0], LONGREAL),
      
      ax = FLOAT(a.c[1], LONGREAL),
      bx = FLOAT(b.c[1], LONGREAL),
      dx = ax*aw - bx*bw,
      
      ay = FLOAT(a.c[2], LONGREAL),
      by = FLOAT(b.c[2], LONGREAL),
      dy = ay*aw - by*bw,
      
      az = FLOAT(a.c[3], LONGREAL),
      bz = FLOAT(b.c[3], LONGREAL),
      dz = az*aw - bz*bw
    DO
      RETURN dx * dx + dy * dy + dz * dz
    END;
  END DistSqr;

PROCEDURE PerspMap(READONLY obs, foc, upp: Point): PMap =
  VAR m: PMap;
      r, s, t: R3.T;
      d, uno: REAL;
      mt, mr, mc, mrt: R4x4.T;
  BEGIN
    <* ASSERT foc.c[0] > 0.0 *>  (* Focus must be finite and hither *)
    <* ASSERT obs.c[0] >= 0.0 *> (* Observer must be hither or infinite *)
    <* ASSERT upp.c[0] >= 0.0 *> (* Zenith must be hither or infinite *)

    (* Compute translation matrix *)
    FOR i := 0 TO 3 DO
      FOR j := 0 TO 3 DO
        IF i = j THEN mt[i,j] := foc.c[0];
        ELSIF i = 0 THEN mt[i,j] := -foc.c[j];
        ELSE mt[i,j] := 0.0;
        END;
      END;
    END;

    (* Compute rotation matrix *)
    t := Dir(foc, obs);
    s := R3.Orthize(Dir(foc, upp), t);
    (* Zenith reference point (upp) must not be on imagesys Z axis: *)
    <* ASSERT NOT R3.NormSqr(s) < 1.0d-12 *> 
    s := R3.Dir(s);
    r := R3.Dir(R3Extras.Cross(s, t));
    
    mr[0,0] := 1.0;
    FOR i := 1 TO 3 DO
        mr[0,i] := 0.0;
        mr[i,0] := 0.0;
        mr[i,1] := r[i-1];
        mr[i,2] := s[i-1];
        mr[i,3] := t[i-1];
    END;

    (* Compose the two matrices: *)
    mrt := R4x4.Mul(mt, mr);

    IF obs.c[0] = 0.0 THEN
      (* Observer is at infinity; cilindrical projection. *)
      m.dir := mrt;
    ELSE
      (* Observer is finite; add conical projection step. *)
      d := FLOAT(R3.Dist(ToCartesian(obs), ToCartesian(foc)));
      IF ABS(d) > 1.0 THEN
        uno := -1.0/d;
        d := 1.0;
      ELSE
        uno := -1.0;
      END;
      FOR i := 0 TO 3 DO
        FOR j := 0 TO 3 DO
          IF i = j THEN mc[i,j] := d ELSE mc[i,j] := 0.0 END;
        END;
      END;
      mc[3,0] := uno;
      m.dir := R4x4.Mul(mrt, mc);
    END;
    
    (* Compute inverse matrix: *)
    m.inv := R4x4.Inv(m.dir);
    RETURN m;
  END PerspMap;

BEGIN 
END HR3.
