MODULE R4x4 EXPORTS R4x4, R4x4Extras;
(*
  Hand-optimized implementation of "R4x4" and "R4x4Extras". 

  Created by J. Stolfi, with contributions by M. C. Carrard.  *)

IMPORT R4Rep, R4x4Rep, R4Extras, RMxN, Wr, Thread;

TYPE RowT = ARRAY [0..N-1] OF ElemT;

PROCEDURE Null(): T =
  BEGIN
    RETURN T{RowT{0.0, ..}, ..}
  END Null;
  
PROCEDURE Identity(): T =
  BEGIN
    RETURN T{
      RowT{1.0, 0.0, 0.0, 0.0},
      RowT{0.0, 1.0, 0.0, 0.0},
      RowT{0.0, 0.0, 1.0, 0.0},
      RowT{0.0, 0.0, 0.0, 1.0}
    }
  END Identity;
    
PROCEDURE ToReal(READONLY m: T): RealT =
  BEGIN
    RETURN m
  END ToReal;
  
PROCEDURE ToLongReal(READONLY m: T): LongRealT =
  VAR lm: LongRealT;
  BEGIN
    FOR i := 0 TO N-1 DO
      FOR j := 0 TO N-1 DO
        lm[i,j] := FLOAT(m[i,j], LONGREAL)
      END
    END;
    RETURN lm
  END ToLongReal;
    
PROCEDURE FromReal(READONLY c: RealT): T =
  BEGIN
    RETURN c
  END FromReal;

PROCEDURE FromLongReal(READONLY c: LongRealT): T =
  VAR m: T;
  BEGIN
    FOR i := 0 TO N-1 DO
      FOR j := 0 TO N-1 DO
        m[i,j] := FLOAT(c[i,j], ElemT)
      END
    END;
    RETURN m
  END FromLongReal;

PROCEDURE Get(READONLY m: T; i, j: [0..N-1]): LONGREAL =
  BEGIN
    RETURN FLOAT(m[i, j], LONGREAL)
  END Get;
    
PROCEDURE Set(VAR m: T; i, j: [0..N-1]; x: LONGREAL) =
  BEGIN
    m[i, j] := FLOAT(x, ElemT)
  END Set;

PROCEDURE MapRow(READONLY a: R4Rep.T; READONLY m: T): R4Rep.T =
  VAR r: R4Rep.T;
  BEGIN
    WITH
      a0 = FLOAT(a[0], LONGREAL), 
      a1 = FLOAT(a[1], LONGREAL), 
      a2 = FLOAT(a[2], LONGREAL), 
      a3 = FLOAT(a[3], LONGREAL)
    DO
      WITH  
        m00 = FLOAT(m[0,0], LONGREAL), 
        m10 = FLOAT(m[1,0], LONGREAL), 
        m20 = FLOAT(m[2,0], LONGREAL), 
        m30 = FLOAT(m[3,0], LONGREAL) 
      DO
        r[0] := FLOAT(a0*m00 + a1*m10 + a2*m20 + a3*m30)
      END;
      WITH 
        m01 = FLOAT(m[0,1], LONGREAL),
        m11 = FLOAT(m[1,1], LONGREAL),
        m21 = FLOAT(m[2,1], LONGREAL),
        m31 = FLOAT(m[3,1], LONGREAL)
      DO
        r[1] := FLOAT(a0*m01 + a1*m11 + a2*m21 + a3*m31)
      END;
      WITH
        m02 = FLOAT(m[0,2], LONGREAL),
        m12 = FLOAT(m[1,2], LONGREAL),
        m22 = FLOAT(m[2,2], LONGREAL),
        m32 = FLOAT(m[3,2], LONGREAL)
      DO
        r[2] := FLOAT(a0*m02 + a1*m12 + a2*m22 + a3*m32)
      END;
      WITH
        m03 = FLOAT(m[0,3], LONGREAL),
        m13 = FLOAT(m[1,3], LONGREAL),
        m23 = FLOAT(m[2,3], LONGREAL),
        m33 = FLOAT(m[3,3], LONGREAL)
      DO
        r[3] := FLOAT(a0*m03 + a1*m13 + a2*m23 + a3*m33)
      END
    END;
    RETURN r
  END MapRow;

PROCEDURE MapCol(READONLY m: T; READONLY a: R4Rep.T): R4Rep.T =
  VAR r: R4Rep.T;
  BEGIN
    WITH
      a0 = FLOAT(a[0], LONGREAL),
      a1 = FLOAT(a[1], LONGREAL),
      a2 = FLOAT(a[2], LONGREAL),
      a3 = FLOAT(a[3], LONGREAL)
    DO
      WITH 
        m00 = FLOAT(m[0,0], LONGREAL), 
        m01 = FLOAT(m[0,1], LONGREAL), 
        m02 = FLOAT(m[0,2], LONGREAL), 
        m03 = FLOAT(m[0,3], LONGREAL)
      DO
        r[0] := FLOAT(m00*a0 + m01*a1 + m02*a2 + m03*a3)
      END;
      WITH  
        m10 = FLOAT(m[1,0], LONGREAL), 
        m11 = FLOAT(m[1,1], LONGREAL), 
        m12 = FLOAT(m[1,2], LONGREAL), 
        m13 = FLOAT(m[1,3], LONGREAL)
      DO
        r[1] := FLOAT(m10*a0 + m11*a1 + m12*a2 + m13*a3)
      END;
      WITH
        m20 = FLOAT(m[2,0], LONGREAL), 
        m21 = FLOAT(m[2,1], LONGREAL), 
        m22 = FLOAT(m[2,2], LONGREAL), 
        m23 = FLOAT(m[2,3], LONGREAL)
      DO
        r[2] := FLOAT(m20*a0 + m21*a1 + m22*a2 + m23*a3)
      END;
      WITH
        m30 = FLOAT(m[3,0], LONGREAL), 
        m31 = FLOAT(m[3,1], LONGREAL), 
        m32 = FLOAT(m[3,2], LONGREAL), 
        m33 = FLOAT(m[3,3], LONGREAL)
      DO
        r[3] := FLOAT(m30*a0 + m31*a1 + m32*a2 + m33*a3)
      END
    END;
    RETURN r
  END MapCol;

PROCEDURE Mul(READONLY m, n: T): T =
  VAR r: T;
  BEGIN
    FOR i := 0 TO N-1 DO
      r[i] := MapRow(m[i], n)
    END;
    RETURN r
  END Mul;

PROCEDURE Adj(READONLY m: T): T =
  BEGIN
    WITH
      m00 = FLOAT(m[0,0], LONGREAL),
      m01 = FLOAT(m[0,1], LONGREAL),
      m02 = FLOAT(m[0,2], LONGREAL),
      m03 = FLOAT(m[0,3], LONGREAL),
      
      m10 = FLOAT(m[1,0], LONGREAL),
      m11 = FLOAT(m[1,1], LONGREAL),
      m12 = FLOAT(m[1,2], LONGREAL),
      m13 = FLOAT(m[1,3], LONGREAL),
      
      m20 = FLOAT(m[2,0], LONGREAL),
      m21 = FLOAT(m[2,1], LONGREAL),
      m22 = FLOAT(m[2,2], LONGREAL),
      m23 = FLOAT(m[2,3], LONGREAL),
      
      m30 = FLOAT(m[3,0], LONGREAL),
      m31 = FLOAT(m[3,1], LONGREAL),
      m32 = FLOAT(m[3,2], LONGREAL),
      m33 = FLOAT(m[3,3], LONGREAL),
      
      m01_01 = m00 * m11 - m01 * m10,
      m01_02 = m00 * m12 - m02 * m10,
      m01_12 = m01 * m12 - m02 * m11,
      m01_03 = m00 * m13 - m03 * m10,
      m01_13 = m01 * m13 - m03 * m11,
      m01_23 = m02 * m13 - m03 * m12,

      m23_01 = m20 * m31 - m21 * m30,
      m23_02 = m20 * m32 - m22 * m30,
      m23_12 = m21 * m32 - m22 * m31,
      m23_03 = m20 * m33 - m23 * m30,
      m23_13 = m21 * m33 - m23 * m31,
      m23_23 = m22 * m33 - m23 * m32,
      
      m012_012 = m01_01 * m22 - m01_02 * m21 + m01_12 * m20,
      m012_013 = m01_01 * m23 - m01_03 * m21 + m01_13 * m20,
      m012_023 = m01_02 * m23 - m01_03 * m22 + m01_23 * m20,
      m012_123 = m01_12 * m23 - m01_13 * m22 + m01_23 * m21,
      
      m013_012 = m01_01 * m32 - m01_02 * m31 + m01_12 * m30,
      m013_013 = m01_01 * m33 - m01_03 * m31 + m01_13 * m30,
      m013_023 = m01_02 * m33 - m01_03 * m32 + m01_23 * m30,
      m013_123 = m01_12 * m33 - m01_13 * m32 + m01_23 * m31,
      
      m023_012 = m00 * m23_12 - m01 * m23_02 + m02 * m23_01,
      m023_013 = m00 * m23_13 - m01 * m23_03 + m03 * m23_01,
      m023_023 = m00 * m23_23 - m02 * m23_03 + m03 * m23_02,
      m023_123 = m01 * m23_23 - m02 * m23_13 + m03 * m23_12,
      
      m123_012 = m10 * m23_12 - m11 * m23_02 + m12 * m23_01,
      m123_013 = m10 * m23_13 - m11 * m23_03 + m13 * m23_01,
      m123_023 = m10 * m23_23 - m12 * m23_03 + m13 * m23_02,
      m123_123 = m11 * m23_23 - m12 * m23_13 + m13 * m23_12
      
    DO
      RETURN T{
        R4Rep.T{FLOAT(+m123_123), FLOAT(-m023_123), FLOAT(+m013_123), FLOAT(-m012_123)},
        R4Rep.T{FLOAT(-m123_023), FLOAT(+m023_023), FLOAT(-m013_023), FLOAT(+m012_023)},
        R4Rep.T{FLOAT(+m123_013), FLOAT(-m023_013), FLOAT(+m013_013), FLOAT(-m012_013)},
        R4Rep.T{FLOAT(-m123_012), FLOAT(+m023_012), FLOAT(-m013_012), FLOAT(+m012_012)}
      }
    END;
  END Adj;

PROCEDURE Inv(READONLY m: T): T =
  BEGIN
    WITH
      m00 = FLOAT(m[0,0], LONGREAL),
      m01 = FLOAT(m[0,1], LONGREAL),
      m02 = FLOAT(m[0,2], LONGREAL),
      m03 = FLOAT(m[0,3], LONGREAL),
      
      m10 = FLOAT(m[1,0], LONGREAL),
      m11 = FLOAT(m[1,1], LONGREAL),
      m12 = FLOAT(m[1,2], LONGREAL),
      m13 = FLOAT(m[1,3], LONGREAL),
      
      m20 = FLOAT(m[2,0], LONGREAL),
      m21 = FLOAT(m[2,1], LONGREAL),
      m22 = FLOAT(m[2,2], LONGREAL),
      m23 = FLOAT(m[2,3], LONGREAL),
      
      m30 = FLOAT(m[3,0], LONGREAL),
      m31 = FLOAT(m[3,1], LONGREAL),
      m32 = FLOAT(m[3,2], LONGREAL),
      m33 = FLOAT(m[3,3], LONGREAL),
      
      m01_01 = m00 * m11 - m01 * m10,
      m01_02 = m00 * m12 - m02 * m10,
      m01_12 = m01 * m12 - m02 * m11,
      m01_03 = m00 * m13 - m03 * m10,
      m01_13 = m01 * m13 - m03 * m11,
      m01_23 = m02 * m13 - m03 * m12,

      m23_01 = m20 * m31 - m21 * m30,
      m23_02 = m20 * m32 - m22 * m30,
      m23_12 = m21 * m32 - m22 * m31,
      m23_03 = m20 * m33 - m23 * m30,
      m23_13 = m21 * m33 - m23 * m31,
      m23_23 = m22 * m33 - m23 * m32,
      
      m012_012 = m01_01 * m22 - m01_02 * m21 + m01_12 * m20,
      m012_013 = m01_01 * m23 - m01_03 * m21 + m01_13 * m20,
      m012_023 = m01_02 * m23 - m01_03 * m22 + m01_23 * m20,
      m012_123 = m01_12 * m23 - m01_13 * m22 + m01_23 * m21,
      
      m013_012 = m01_01 * m32 - m01_02 * m31 + m01_12 * m30,
      m013_013 = m01_01 * m33 - m01_03 * m31 + m01_13 * m30,
      m013_023 = m01_02 * m33 - m01_03 * m32 + m01_23 * m30,
      m013_123 = m01_12 * m33 - m01_13 * m32 + m01_23 * m31,
      
      m023_012 = m00 * m23_12 - m01 * m23_02 + m02 * m23_01,
      m023_013 = m00 * m23_13 - m01 * m23_03 + m03 * m23_01,
      m023_023 = m00 * m23_23 - m02 * m23_03 + m03 * m23_02,
      m023_123 = m01 * m23_23 - m02 * m23_13 + m03 * m23_12,
      
      m123_012 = m10 * m23_12 - m11 * m23_02 + m12 * m23_01,
      m123_013 = m10 * m23_13 - m11 * m23_03 + m13 * m23_01,
      m123_023 = m10 * m23_23 - m12 * m23_03 + m13 * m23_02,
      m123_123 = m11 * m23_23 - m12 * m23_13 + m13 * m23_12,
      
      d = m012_012 * m33 - m012_013 * m32 + m012_023 * m31 - m012_123 * m30
    DO
      RETURN T{
        R4Rep.T{FLOAT(+m123_123/d), FLOAT(-m023_123/d), FLOAT(+m013_123/d), FLOAT(-m012_123/d)},
        R4Rep.T{FLOAT(-m123_023/d), FLOAT(+m023_023/d), FLOAT(-m013_023/d), FLOAT(+m012_023/d)},
        R4Rep.T{FLOAT(+m123_013/d), FLOAT(-m023_013/d), FLOAT(+m013_013/d), FLOAT(-m012_013/d)},
        R4Rep.T{FLOAT(-m123_012/d), FLOAT(+m023_012/d), FLOAT(-m013_012/d), FLOAT(+m012_012/d)}
      }
    END
  END Inv;

PROCEDURE Det(READONLY m: T): LONGREAL =
  BEGIN
    RETURN R4Extras.Det(m[0], m[1], m[2], m[3])
  END Det;  

PROCEDURE Print(
    wr: Wr.T; 
    READONLY m: T; 
    olp := "( "; osep := "\n  "; orp := " )";
    ilp := "("; isep := " "; irp := ")";
    fmt: PROCEDURE (x: ElemT): TEXT := R4x4Rep.DefaultElemFmt;
    align: BOOLEAN := TRUE;
  ) RAISES {Wr.Failure, Thread.Alerted} =
  BEGIN
    RMxN.Print(wr, m, olp, osep, orp, ilp, isep, irp, fmt, align)
  END Print;

PROCEDURE ToText(READONLY m: T): TEXT =
  BEGIN
    RETURN RMxN.ToText(m)
  END ToText;

BEGIN 
END R4x4.
