MODULE Map;

IMPORT Oct, Triang, Color, Word;
FROM Oct IMPORT FRBits, Sym, Flip, Splice, Oprev, Onext, Lnext, Rot;
FROM Triang IMPORT OrgV, LeftF, SetLeft, SetAllOrgs;
                
REVEAL 
  Edge = PublicEdge BRANDED OBJECT
    order: CARDINAL;
    ca: ARRAY FRBits OF Triang.Arc;
    mid: Triang.Arc; 
  OVERRIDES
    init := MapInit;
  END;

PROCEDURE MapInit(s: Edge; order: CARDINAL): Edge =
  BEGIN
    EVAL NARROW(s, Oct.Edge).init();  
    s.ca := Triang.MakeGrid(order, order);
    s.order := order;
    s.mid := GetMiddle(Oprev(s.ca[0]), order);
    RETURN s
  END MapInit;

PROCEDURE MakeEdge(order: CARDINAL): Arc =
  BEGIN  
    WITH e = NEW(Edge).init(order) DO
      RETURN Arc{edge := e, bits := 0};
    END;
  END MakeEdge;

PROCEDURE SetPrimalProperties(
    a: Arc; 
    vertexColor: Color.T;
    vertexRadius: REAL;
    edgeColor: Color.T;
    edgeRadius: REAL;
    faceColor: Color.T;
    faceTransp: Color.T;
  ) =
  BEGIN
    FOR k := 0 TO 1 DO
      WITH 
        r = a.bits,
        e = NARROW(a.edge, Edge),
        num = e.num
      DO
        Triang.SetQuarterGridProperties(
          c := e.ca[r], 
          order := e.order,
          vertexColor := vertexColor,
          vertexRadius := vertexRadius,
          edgeColor := edgeColor,
          edgeRadius := edgeRadius,
          facePatch := 4 * num + r,
          faceColor := faceColor,
          faceTransp := faceTransp
        );
        a := Sym(a)
      END
    END
  END SetPrimalProperties;
  
PROCEDURE Corner(a: Arc): Triang.Arc =
  BEGIN
    WITH r = a.bits DO
      RETURN NARROW(a.edge, Edge).ca[r];
    END;
  END Corner;

PROCEDURE SetCorner(a: Arc; c: Triang.Arc) =
  BEGIN
    WITH r = a.bits DO
      NARROW(a.edge, Edge).ca[r] := c;
    END;
  END SetCorner;

PROCEDURE CCorner(a: Arc): Oct.Arc =
  BEGIN
    WITH r = Word.Xor(a.bits, 1) DO
      RETURN Flip(NARROW(a.edge, Edge).ca[r]);
    END;
  END CCorner;

PROCEDURE SetCCorner(a: Arc; c: Triang.Arc) =
  BEGIN
    WITH r = Word.Xor(a.bits, 1) DO
      NARROW(a.edge, Edge).ca[r] := Flip(c);
    END;
  END SetCCorner;

PROCEDURE Middle(a: Arc): Oct.Arc =
  BEGIN
    RETURN NARROW(a.edge, Edge).mid;
  END Middle;

PROCEDURE GetMiddle(c: Oct.Arc; order: CARDINAL): Oct.Arc =
  (*
    Returns the middle arc on the diagonal spine of an unglued
    grid, given the first arc "c" of the diagonal, and 
    the grid "order". This arc should not be affected by GluePatch
    or Consist if the grid order is 3 or more.  *)
  VAR ct: Oct.Arc;
  BEGIN
    ct := Sym(c);
    FOR k := 1 TO order-1 DO 
      ct := Onext(Onext(ct));
      IF k MOD 2 = 0 THEN ct := Onext(Onext(ct)) END;
      ct := Sym(ct);
    END;
    RETURN ct;
  END GetMiddle;

PROCEDURE GluePatch(a: Arc) =

  PROCEDURE DeleteCell(aa: Oct.Arc) =
    (*
      Deletes a grid corner cell, in which two adjacent sides have
      been glued together.  Arc "aa" is this common side; Org(a) is 
      a corner of the patch.
    *)
    BEGIN
      WITH
        rr = Oprev(Oprev(Sym(aa))),
        ss = Onext(Onext(Sym(aa))),
        u = OrgV(aa),
        f = LeftF(ss)
      DO
        <* ASSERT OrgV(rr) = OrgV(ss) *>
        <* ASSERT OrgV(Sym(rr)) = OrgV(Sym(ss)) *>
        <* ASSERT Onext(Onext(Sym(ss))) = Sym(rr) *>
        Splice(rr, ss);
        Splice(Oprev(Sym(rr)), Oprev(Sym(ss)));
        SetLeft(rr, f);
        SetAllOrgs(Sym(rr), u)
      END;
    END DeleteCell;

  VAR ac, bc, ad: Oct.Arc;
  BEGIN
    WITH
      b = Onext(a),
      ao = NARROW(a.edge, Edge),
      bo = NARROW(b.edge, Edge),
      order = ao.order
    DO
      <* ASSERT ao.order = bo.order *>
      ac := Corner(a);
      bc := CCorner(b);

      IF (ac # bc) THEN
        (* Not yet glued, glue it: *)
        
        (* Save edges to be glued: *)
        ad := ac;
        FOR i := 1 TO order-1 DO ad := Lnext(ad) END;
        ad := Sym(ad);

        (* Glue patch sides: *)
        WITH al = Triang.Glue(ac, bc, order) DO
          <* ASSERT Sym(al) = ad *>
          (* Update Corner(a) and CCorner(b) to mark (a,b) as glued *)
          SetCCorner(b, ac);
          SetCorner(Rot(b), ad)
        END;

        (* Eliminate degenerate triangles: *) 
        IF b = a THEN DeleteCell(ac) END;
        IF b = Sym(a) THEN DeleteCell(ad) END
      END;
    END;
  END GluePatch;

BEGIN
END Map.
