MODULE MixedEnergy;
IMPORT Fmt, Wr, Thread, TextWr, Energy, LR3; 
FROM Triang IMPORT Topology;
FROM Energy IMPORT Coords, Gradient;

REVEAL
  T = Public BRANDED OBJECT
      ekDc: REF ARRAY OF LR3.T;      (* (Work) Gradient of some ingredient *)
    OVERRIDES
      init := Init;
      defTop := DefTop;
      defVar := DefVar;
      eval := Eval;
      name := Name;
    END;
    
PROCEDURE Init(
    erg: T;
    READONLY term: ARRAY OF Energy.T; 
    READONLY weight: ARRAY OF REAL
  ): T =
  BEGIN
    WITH
      NT = NUMBER(term)
    DO
      <* ASSERT NUMBER(weight) = NT *>
      erg.term := NEW(REF ARRAY OF Energy.T, NT);
      erg.weight := NEW(REF ARRAY OF REAL, NT);
      erg.termValue := NEW(REF ARRAY OF LONGREAL, NT);
      erg.term^ := term;
      erg.weight^ := weight;
      RETURN erg
    END
  END Init;

PROCEDURE DefTop(erg: T; READONLY top: Topology) =
  BEGIN
    WITH 
      term = erg.term^,
      NT = NUMBER(term)
    DO
      erg.ekDc := NEW(REF ARRAY OF LR3.T, top.NV);
      FOR k := 0 TO NT-1 DO term[k].defTop(top) END
    END;
  END DefTop;

PROCEDURE DefVar(erg: T; READONLY variable: ARRAY OF BOOLEAN) =
  BEGIN
    WITH 
      term = erg.term^,
      NT = NUMBER(term)
    DO
      FOR k := 0 TO NT-1 DO term[k].defVar(variable) END
    END;
  END DefVar;

PROCEDURE Eval(
    erg: T; 
    READONLY c: Coords; 
    VAR e: LONGREAL; 
    grad: BOOLEAN;
    VAR eDc: Gradient;
  ) =
  BEGIN
    WITH 
      term = erg.term^,
      weight = erg.weight^,
      termValue = erg.termValue^,
      ekDc = erg.ekDc^,
      NT = NUMBER(term),
      NV = NUMBER(c)
    DO
      <* ASSERT NUMBER(eDc) = NV *>
      e := 0.0d0;
      FOR v := 0 TO NV-1 DO eDc[v] := LR3.T{0.0d0, ..} END;
      FOR k := 0 TO NT-1 DO
        term[k].eval(c, termValue[k], grad, ekDc);
        e := e + FLOAT(weight[k], LONGREAL) * termValue[k];
        IF grad THEN
          FOR v := 0 TO NV-1 DO 
            eDc[v] := LR3.Mix(1.0d0, eDc[v], FLOAT(weight[k], LONGREAL), ekDc[v])
          END
        END
      END
    END;
  END Eval;

PROCEDURE Name(erg: T): TEXT =
  <* FATAL Wr.Failure, Thread.Alerted *>
  VAR wr := NEW(TextWr.T).init();
  BEGIN
    WITH 
      term = erg.term^,
      weight = erg.weight^,
      NT = NUMBER(term)
    DO
      Wr.PutText(wr, "Mixed(");
      FOR i := 0 TO NT-1 DO 
        IF i > 0 THEN Wr.PutText(wr, ", ") END;
        Wr.PutText(wr, "[");
        Wr.PutText(wr, Fmt.Int(i));
        Wr.PutText(wr, "] = ");
        Wr.PutText(wr, Fmt.Real(weight[i], Fmt.Style.Fix, 4));
        Wr.PutText(wr, "");
        Wr.PutText(wr, term[i].name())
      END;
      Wr.PutText(wr, ")")
    END;
    RETURN TextWr.ToText(wr)
  END Name;

BEGIN
END MixedEnergy.
