MODULE ParseEnergyParams;
IMPORT ParseParams;
IMPORT 
  MixedEnergy, Energy, 
  (* ElastEnergy, RelarEnergy, *)
  ElectEnergy, PotenEnergy, PAreaEnergy, CurvEnergy,
  CornerEnergy, ExcenEnergy;

PROCEDURE Parse(pp: ParseParams.T): MixedEnergy.T RAISES {ParseParams.Error} =
  CONST MaxTerms = 50;
  VAR e: ARRAY [0..MaxTerms-1] OF Energy.T;
      w: ARRAY [0..MaxTerms-1] OF REAL;
      n: CARDINAL := 0;
  BEGIN

    WHILE pp.keywordPresent("-energy") DO 
      w[n] := pp.getNextReal(0.0, 1000.0);
      IF pp.testNext("Curv") THEN
        e[n] := NEW(CurvEnergy.T).init();
      ELSIF pp.testNext("Corner") THEN
        e[n] := NEW(CornerEnergy.T).init();
      ELSIF pp.testNext("Elect") THEN
        VAR fuzz: REAL;
        BEGIN
          IF pp.testNext("fuzz") THEN
            fuzz := pp.getNextReal(0.0, 10.0);
          ELSE
            fuzz := 0.5;
          END;
          e[n] := NEW(ElectEnergy.T, fuzz := fuzz)
        END;
      ELSIF pp.testNext("PArea") THEN
        VAR area: REAL;
        BEGIN
          IF pp.testNext("area") THEN
            area := pp.getNextReal(0.0001, 10000.0);
          ELSE
            area := 1.0;
          END;
          e[n] := NEW(PAreaEnergy.T, area := area).init()
        END;
      ELSIF pp.testNext("Excen") THEN
        e[n] := NEW(ExcenEnergy.T).init();
      ELSIF pp.testNext("Poten") THEN
        e[n] := NEW(PotenEnergy.T).init(); 
      ELSE
        pp.error("Unknown energy: \"" & pp.getNext() & "\"")
      END;
      IF w[n] # 0.0 THEN INC(n) END;
    END;

    (* Default energy: *)
    IF n = 0 THEN
      RETURN NIL
    ELSE
      RETURN NEW(MixedEnergy.T).init(SUBARRAY(e, 0, n), SUBARRAY(w, 0, n))
    END;
  END Parse;
  
BEGIN
END ParseEnergyParams.



