INTERFACE DownMinimizer;

(*
  Multivariate function minimization by downhill jumping.
  
  Created 96/01/06 by J.Stolfi
*)

IMPORT Minimizer;

TYPE
  T <: Public;
    (*
      A simple gradient-based optimizer.
      
      Each new probe point is generated from the current minimum by
      moving a certain distance (the "step size") in the direction
      opposite to the gradient.  If the new point is better than the
      previous one, it is accepted and the step size gets multiplied
      by "alpha".  Otherwise the new point is rejected and the step
      size gets multiplied by "beta".
    *)

  Public = Minimizer.T OBJECT
    METHODS
        
      setAlphaBeta(alpha, beta: LONGREAL): T;
        (*
          Specifies the step adjustment factors "alpha" and "beta".
          Defaults are "alpha = 2**\phi = 3.06956450765297882", "beta = 2**-1 = 0.5"*)
        
    END;
    
END DownMinimizer.
