MODULE EulerIntegrator;

IMPORT Math, Integrator;
FROM Math IMPORT sqrt;
FROM Integrator IMPORT DiffProc, Time, Coord, State, Velocity, Error;

REVEAL T = Public BRANDED OBJECT
    v, v2: REF Velocity;
    sb: REF State;
    er: REF Error;
  OVERRIDES
    name := Name;
    setSize := SetSize;
    computeStep := ComputeStep;
    integrate := Integrate;
  END;

PROCEDURE Name(<*UNUSED*> ei: T): TEXT =
  BEGIN
    RETURN "EulerIntegrator"
  END  Name;

PROCEDURE SetSize(ei: T; n: CARDINAL): Integrator.T =
  BEGIN
    ei.v := NEW(REF Velocity, n);
    ei.sb := NEW(REF State, n);
    ei.er := NEW(REF Error, n);
    ei.v2 := NEW(REF Velocity, n);
    RETURN ei
  END SetSize;

PROCEDURE Integrate(
    int: T;
    t: Time;
    VAR s: State;
    diff: DiffProc;
    dt: Time;
    checkStep: Integrator.CheckStepProc;
    checkState: Integrator.CheckStateProc := NIL;
  ): Time =
  VAR tb, tf: Time;
  BEGIN
    WITH
      v = int.v^,
      sb = int.sb^,
      er = int.er^,
      v2 = int.v2^
    DO
      tf := t + dt;
      WHILE t < tf DO
        diff(t, s, v);
        tf := MIN(tf, checkState(t, s, v));
        IF tf <= t THEN RETURN t END;
        (* Tries to advance until some time "t" in "(t __ tf]": *)
        REPEAT
          tb := tf;
          Step(diff, t, s, v, tb, sb, er, v2);
          tf := checkStep(t, s, v, tb, sb, er);
          IF tf <= t THEN RETURN t END
        UNTIL tf >= tb;
        t := tb; 
        s := sb;
      END;
    END;
    RETURN t
  END Integrate;

PROCEDURE ComputeStep(<*UNUSED*> int: T; dt: Time; error, tol: Coord): Time =
  BEGIN
    IF error > 0.0d0 THEN
      WITH delta = sqrt(tol/error) DO
        IF delta <= 0.1d0  THEN dt := 0.1d0 * dt
        ELSIF delta >= 10.0d0  THEN dt := 10.0d0 * dt
        ELSE dt := delta * dt
        END
      END
    ELSE
      dt := 10.0d0 * dt;
    END;
    RETURN dt
  END ComputeStep;

PROCEDURE Step(
    diff: DiffProc;
    ta: Time;
    READONLY sa: State;
    READONLY va: Velocity;
    tb: Time;
    VAR sb: State; (* Final state *)
    VAR er: Error; (* Error estimate *)
    VAR v2: Velocity; (* Auxiliary velocities: *)
  ) =
  BEGIN
    WITH
      n = NUMBER(sa),
      dt = tb - ta
    DO
      (* Degree 1 extrapolation: *)
      FOR i := 0 TO n-1 DO
        sb[i] := sa[i] + dt * va[i];
      END;

      (* Error estimation (hack --- shoutd check the thory and do it right!): *)
      diff(ta + dt, sb, v2);
      FOR i := 0 TO n-1 DO
        er[i] := 0.5d0 * dt * (va[i] - v2[i])
      END;
    END
  END Step;

BEGIN 
END EulerIntegrator.
 
