MODULE ParseMinimizerParams;
IMPORT ParseParams;
IMPORT 
  Minimizer, Integrator, UniMin,
  (* PraxisMinimizer, NMSimplexMinimizer, *)
  GradMinimizer, CoordMinimizer, 
  BrentUniMin, JSUniMin,
  EulerIntegrator, RKF4Integrator;

PROCEDURE Parse(pp: ParseParams.T): Minimizer.T RAISES {ParseParams.Error} =
  BEGIN
    IF pp.keywordPresent("-minimizer") THEN
      IF pp.testNext("Grad") THEN
        VAR i: Integrator.T;
        BEGIN
          IF pp.testNext("RKF4") THEN
            i := NEW(RKF4Integrator.T)
          ELSIF pp.testNext("Euler") THEN
            i := NEW(EulerIntegrator.T)
          ELSE
            i := NEW(RKF4Integrator.T)
          END;
          RETURN NEW(GradMinimizer.T).setIntegrator(i)
        END
      ELSIF pp.testNext("Coord") THEN
        VAR i: UniMin.T; n: CARDINAL;
        BEGIN
          IF pp.testNext("Brent") THEN
            i := NEW(BrentUniMin.T)
          ELSIF pp.testNext("JS") THEN
            i := NEW(JSUniMin.T)
          ELSE
            i := NEW(BrentUniMin.T)
          END;
          IF pp.testNext("budget") THEN
            n := pp.getNextInt(5, 100)
          ELSE
            n := 7
          END;
          RETURN NEW(CoordMinimizer.T).setUniMin(i).setBudget(n)
        END
      ELSE
        pp.error("Unknown optimization method: " & pp.getNext());
        <* ASSERT FALSE *>
      END;
    ELSE
      RETURN NIL
    END;
  END Parse;
  
BEGIN
END ParseMinimizerParams.
