MODULE PSPlotTest EXPORTS Main;

IMPORT PSPlot;
FROM PSPlot IMPORT LetterXSize, LetterYSize, Color, Invisible;

PROCEDURE DoIt() =
  BEGIN
    DoEPSTests();
    DoPSTests()
  END DoIt;
  
PROCEDURE DoEPSTests() =
  BEGIN
    WITH
      eps = NEW(PSPlot.EPSFile).open(
        "psplottest.eps", 
        xSize := 160.0d0, 
        ySize := 120.0d0
      )
    DO
      DrawThings(eps);
      eps.close();
    END
  END DoEPSTests;
      
PROCEDURE DoPSTests() =
  BEGIN
    WITH
      ps = NEW(PSPlot.PSFile).open("psplottest.ps")
    DO
      ps.beginPage();
        ps.beginDrawing(xSize := 160.0d0, ySize := 120.0d0);
          DrawThings(ps);
          ps.setTextColor(Color{0.0, 0.0, 1.0});
          ps.caption("First Line Centered\nSecond Line Centered Too", xAlign := 0.5);
          ps.caption("Third Line Left Aligned", xAlign := 0.0);
          ps.caption("Fourth Line Right Aligned", xAlign := 1.0);
          ps.caption("Fifth Line In Large Font", font := "Times-Roman", size := 18.0);
          ps.setTextColor(Color{1.0, 0.0, 0.0});
          ps.caption("Sixth Line In Red", xAlign := 0.5);
        ps.endDrawing();

        ps.beginDrawing(
          xSize := 60.0d0, ySize := 60.0d0,
          xCenter := 70.0d0, yCenter := 70.0d0
        );
          DrawThings(ps);
          ps.caption("This Figure No Caption", xAlign := 0.5);
        ps.endDrawing();
      ps.endPage();
      ps.beginPage();
        ps.beginDrawing(
          xSize := 160.0d0, ySize := 160.0d0,
          xCenter := LetterXSize/2.0d0, yCenter := LetterYSize/2.0d0
        );
          DrawThings(ps);
        ps.endDrawing();
      ps.endPage();
      ps.close()
    END
  END DoPSTests;
  
PROCEDURE DrawThings(f: PSPlot.File) =
  BEGIN
    f.comment("Thick solid red frame:");
    f.setLineColor(Color{1.0, 0.0, 0.0});
    f.setLineWidth(1.0);
    f.setLineSolid();
    f.frame();

    f.comment("Thin dashed red gridlines:");
    f.setLineWidth(0.25);
    f.setLineDashed(ARRAY OF REAL{2.0, 1.0}, 1.0);
    f.setLineColor(Color{1.0, 0.0, 0.0});
    f.setGridSize(20, 40);
    f.gridLines();

    f.setScale(PSPlot.Axis.Y, -10.0d0, 10.0d0);
    FOR i := 0 TO 1 DO
      IF i = 0 THEN
        f.setScale(PSPlot.Axis.X, -10.0d0, 30.0d0)
      ELSE
        f.setScale(PSPlot.Axis.X, -30.0d0, 10.0d0);
      END;

      f.comment("Thin solid black coordinate lines:");
      f.setLineWidth(0.25);
      f.setLineColor(Color{0.0, 0.0, 0.0});
      f.setLineSolid();
      f.coordLine(PSPlot.Axis.X, 0.17d0);
      f.coordLine(PSPlot.Axis.Y, 3.14d0);

      f.comment("Medium solid black segments:");
      f.setLineWidth(0.50);
      f.setLineColor(Color{0.0, 0.0, 0.0});
      f.setLineSolid();
      f.segment(-6.0d0, -6.0d0, +6.0d0, -5.0d0);
      f.segment(-2.0d0, -5.0d0, +2.0d0, -6.0d0);

      f.comment("Medium solid black figures, yellow filled:");
      f.setLineWidth(0.50);
      f.setLineColor(Color{0.0, 0.0, 0.0});
      f.setLineSolid();
      f.setFillColor(Color{1.0, 1.0, 0.0});

      f.gridCell(08, 12);
      f.rectangle(-5.0d0, -4.0d0, -5.0d0, -4.0d0);
      f.circle(-4.5d0, -2.0d0, 5.0);
      f.ellipse(-4.5d0, +2.0d0, +0.5d0, +1.0d0, 00.0d0, +0.5d0);
      f.triangle(+6.0d0, -4.0d0, +7.5d0, -3.5d0, +6.5d0, -2.5d0);

      f.comment("Medium solid red figures, unfilled:");
      f.setFillColor(Invisible);
      f.setLineColor(Color{1.0, 0.0, 0.0});

      f.gridCell(10, 12);
      f.rectangle(-3.0d0, -2.0d0, -5.0d0, -4.0d0);
      f.circle(-2.5d0, -2.0d0, 5.0);
      f.ellipse(-2.5d0, +2.0d0, +0.5d0, +1.0d0, 00.0d0, +0.5d0);
      f.triangle(+2.0d0, -4.0d0, +3.5d0, -3.5d0, +2.5d0, -2.5d0);

      f.comment("Unstroked figures, pink filled:");
      f.setFillColor(Color{1.0, 0.5, 0.5});
      f.setLineColor(Invisible);

      f.gridCell(12, 12);
      f.rectangle(-1.0d0, 00.0d0, -5.0d0, -4.0d0);
      f.circle(-1.5d0, -2.0d0, 5.0);
      f.ellipse(-0.5d0, +2.0d0, +0.5d0, +1.0d0, 00.0d0, +0.5d0);
      f.triangle(+4.0d0, -4.0d0, +5.5d0, -3.5d0, +4.5d0, -2.5d0);

      f.comment("Text in various positions:");
      f.setTextColor(Color{1.0, 0.0, 1.0});
      f.setFillColor(Color{0.0, 0.0, 1.0});
      f.setLineColor(Invisible);

      f.label(
        "magenta C10", -5.0d0, 8.0d0, 0.0, 0.0, 0.0, 
        font := "Courier", size := 10.0
      );
      f.circle(-5.0d0, 8.0d0, 1.0);

      f.label(
        "magenta TR12", -5.0d0, 7.0d0, 0.5, 0.0, 0.0,
        font := "Times-Roman", size := 12.0
      );
      f.circle(-5.0d0, 7.0d0, 1.0);

      f.setTextColor(Color{0.0, 0.5, 1.0});
      f.label(
        "blue H8", -5.0d0, 6.0d0, 0.0, 0.5, 0.0,
        font := "Helvetica", size := 8.0
      );
      f.circle(-5.0d0, 6.0d0, 1.0);

      f.label(
        "blue C12", +5.0d0, 6.0d0, 0.0, 0.5, 60.0,
        font := "Courier", size := 12.0
      );
      f.circle(+5.0d0, 6.0d0, 1.0);
    END;    
  END DrawThings;

BEGIN
  DoIt()
END PSPlotTest.
